﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.Toolkit.Mvvm.Input;
using System.Collections.ObjectModel;
using System.Windows;
using System.Threading;
using Microsoft.Toolkit.Mvvm.ComponentModel;
using BPASmartClient.Device;
using BPASmartClient.Helper;
using BPASmartClient.Business;

namespace BPASmartClient.ViewModel
{
    public class AlarmViewModel : ObservableObject
    {
        public AlarmViewModel()
        {
            ControlCommand = new RelayCommand(() =>
            {
                if (ControlButText == "报警复位")
                {
                    return;
                }
                if (ControlButText == "开始查询")
                {
                    var lists = Sqlite<Alarm>.GetInstance.Base.ToList();
                    var res = lists.Where(p => Convert.ToDateTime(p.Date) >= StartDateTime && Convert.ToDateTime(p.Date) <= EndDateTime).ToList();
                    if (res != null)
                    {
                        HistoryAlarm.Clear();
                        foreach (var item in res)
                        {
                            HistoryAlarm.Add(item);
                        }
                    }
                }

            });

            SwitchCommand = new RelayCommand(() =>
            {
                if (ButContent == "历史报警")
                {
                    GetHistoryAlarm();
                    CurrentDataVis = Visibility.Hidden;
                    HistoryDataVis = Visibility.Visible;
                    IsVisibility = Visibility.Visible;
                    ControlButText = "开始查询";
                    ButContent = "实时报警";
                    return;
                }

                if (ButContent == "实时报警")
                {
                    HistoryDataVis = Visibility.Hidden;
                    CurrentDataVis = Visibility.Visible;
                    IsVisibility = Visibility.Hidden;
                    ControlButText = "报警复位";
                    ButContent = "历史报警";
                    return;
                }
            });

        }


        public static void AlarmMonitoring()
        {
            Plugin.GetInstance()?.GetPlugin<DeviceMgr>()?.GetDevices()?.ForEach(device =>
            {
                device.AddErrorAction = new Action<int, object>((id, o) =>
                  {
                      Application.Current.Dispatcher.Invoke(new Action(() =>
                      {
                          AlarmInfos.Add(new Alarm()
                          {
                              NumId = AlarmInfos.Count,
                              Date = DateTime.Now.ToString("yyyy/MM/dd"),
                              Grade = o?.GetType()?.GetProperty("Type")?.GetValue(o)?.ToString(),
                              Info = o?.GetType()?.GetProperty("Text")?.GetValue(o)?.ToString(),
                              Time = DateTime.Now.ToString("HH:mm:ss"),
                          });
                      }));
                  });

                device.DeleteErrorAction = new Action<int, object>((id, o) =>
                {
                    var info = o?.GetType()?.GetProperty("Text")?.GetValue(o)?.ToString();
                    var res = AlarmInfos.FirstOrDefault(p => p.Info == info);
                    if (res != null) AlarmInfos.Remove(res);
                });
            });



            //ThreadManage.GetInstance.StartLong(new Action(() =>
            //{
            //    if (SimpleFactory.GetInstance.Alarm != null)
            //    {
            //        foreach (var item in SimpleFactory.GetInstance.Alarm.GetType().GetProperties())
            //        {
            //            var res = item.GetValue(SimpleFactory.GetInstance.Alarm);
            //            if (res != null)
            //            {
            //                if (res is bool blen)
            //                {
            //                    if (item.CustomAttributes.Count() > 0)
            //                    {
            //                        if (item.CustomAttributes.ElementAt(0)?.ConstructorArguments.Count() > 0)
            //                        {
            //                            var info = item.CustomAttributes.ElementAt(0)?.ConstructorArguments.ElementAt(0).Value;
            //                            if (info != null)
            //                            {
            //                                App.Current.Dispatcher.Invoke(new Action(() =>
            //                                {
            //                                    AlarmHelper.GetInstance.EdgeAlarm(blen, info.ToString());
            //                                }));
            //                            }
            //                        }
            //                    }
            //                }
            //            }
            //        }
            //    }
            //    Thread.Sleep(500);
            //}), "报警检测监控");
        }

        private void GetHistoryAlarm()
        {
            //var data = Sqlite<Alarm>.GetInstance.Base.ToList();
            //if (data != null)
            //{
            //    AlarmHelper.GetInstance.HistoryAlarms.Clear();
            //    foreach (var item in data)
            //    {
            //        int day = DateTime.Now.Subtract(Convert.ToDateTime(item.Date)).Days;
            //        if (day == 0)
            //        {
            //            AlarmHelper.GetInstance.HistoryAlarms.Add(item);
            //        }
            //    }
            //}
        }

        public RelayCommand SwitchCommand { get; set; }

        public RelayCommand ControlCommand { get; set; }


        public Visibility CurrentDataVis { get { return _mCurrentDataVis; } set { _mCurrentDataVis = value; OnPropertyChanged(); } }
        private Visibility _mCurrentDataVis = Visibility.Visible;


        public Visibility HistoryDataVis { get { return _mHistoryDataVis; } set { _mHistoryDataVis = value; OnPropertyChanged(); } }
        private Visibility _mHistoryDataVis = Visibility.Hidden;


        /// <summary>
        /// 是否显示
        /// </summary>
        public Visibility IsVisibility { get { return _mIsVisibility; } set { _mIsVisibility = value; OnPropertyChanged(); } }
        private Visibility _mIsVisibility = Visibility.Hidden;

        /// <summary>
        /// 文字显示
        /// </summary>
        public string ButContent { get { return _mButContent; } set { _mButContent = value; OnPropertyChanged(); } }
        private string _mButContent = "历史报警";

        /// <summary>
        /// 控制按钮文本显示
        /// </summary>
        public string ControlButText { get { return _mControlButText; } set { _mControlButText = value; OnPropertyChanged(); } }
        private string _mControlButText = "报警复位";


        /// <summary>
        /// 开始时间
        /// </summary>
        public DateTime StartDateTime { get { return _mStartDateTime; } set { _mStartDateTime = value; OnPropertyChanged(); } }
        private DateTime _mStartDateTime = DateTime.Now;

        /// <summary>
        /// 结束时间
        /// </summary>
        public DateTime EndDateTime { get { return _mEndDateTime; } set { _mEndDateTime = value; OnPropertyChanged(); } }
        private DateTime _mEndDateTime = DateTime.Now;


        public static ObservableCollection<Alarm> AlarmInfos { get; set; } = new ObservableCollection<Alarm>();

        public static ObservableCollection<Alarm> HistoryAlarm { get; set; } = new ObservableCollection<Alarm>();

    }
}
